@@ -14,14 +14,14 @@ class ModbusTCPServer(TerminableThread):
     def __init__(self, bind_ifc: str, bind_port: int,
                  data_source: tp.Optional[BaseDataSource] = None,
                  backlog: int = 128):
-        super().__init__()
+        super().__init__(name='accept')
         if data_source is None:
             data_source = TestingDataSource()
-        self.data_source = data_source
         self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         self.socket.bind((bind_ifc, bind_port))
         self.backlog = backlog
-        self.processor = ModbusProcessor(self)
+        self.processor = ModbusProcessor(data_source)
 
     def prepare(self) -> None:
         self.socket.listen(self.backlog)
