@@ -1,7 +1,6 @@
 import struct
-import weakref
 
-from satella.coding import rethrow_as
+from satella.coding import reraise_as
 
 from modbus_tcp_server.data_source import BaseDataSource
 from modbus_tcp_server.datagrams import MODBUSTCPMessage
@@ -13,30 +12,31 @@ STRUCT_HH = struct.Struct('>HH')
 STRUCT_HHB = struct.Struct('>HHB')
 
 
-def read_holding_registers(db: BaseDataSource, unit_id: int, address_start: int, amount: int):
+def read_holding_registers(db: BaseDataSource, unit_id: int,
+                           address_start: int, amount: int) -> bytes:
     output = []
     for address in range(address_start, address_start + amount):
         output.append(db.get_holding_register(unit_id, address))
     output_len = len(output)
-    return struct.pack('>H' + ('H' * output_len), output_len * 2, *output)
+    return struct.pack('>B' + ('H' * output_len), output_len * 2, *output)
 
 
-def read_analog_inputs(db: BaseDataSource, unit_id: int, address_start: int, amount: int):
+def read_analog_inputs(db: BaseDataSource, unit_id: int, address_start: int, amount: int) -> bytes:
     output = []
     for address in range(address_start, address_start + amount):
         output.append(db.get_analog_input(unit_id, address))
     output_len = len(output)
-    return struct.pack('>H' + ('H' * output_len), output_len * 2, *output)
+    return struct.pack('>B' + ('H' * output_len), output_len * 2, *output)
 
 
-def read_coil(db: BaseDataSource, unit_id: int, address_start: int, amount: int):
+def read_coil(db: BaseDataSource, unit_id: int, address_start: int, amount: int) -> bytes:
     bs = BitStream()
     for address in range(address_start, address_start + amount):
         bs.add(db.get_coil(unit_id, address))
     return bytes([len(bs)]) + bytes(bs)
 
 
-def read_discrete_input(db: BaseDataSource, unit_id: int, address_start: int, amount: int):
+def read_discrete_input(db: BaseDataSource, unit_id: int, address_start: int, amount: int) -> bytes:
     bs = BitStream()
     for address in range(address_start, address_start + amount):
         bs.add(db.get_discrete_input(unit_id, address))
@@ -58,7 +58,7 @@ def write_multiple_registers(db: BaseDataSource, unit_id: int, msg: MODBUSTCPMes
     if databytes != 2 * amount:
         raise InvalidFrame('Mismatch between writing amount and no of bytes')
 
-    with rethrow_as(struct.error, InvalidFrame):
+    with reraise_as(struct.error, InvalidFrame):
         for address, value in zip(range(address, address + amount),
                                   struct.unpack('>' + ('H' * amount), reg_data)):
             db.set_holding_register(unit_id, address, value)
@@ -79,6 +79,8 @@ def write_multiple_coils(db: BaseDataSource, unit_id: int, msg: MODBUSTCPMessage
 
 
 TRANSLATION_TABLE = {
+    # Either provide a struct to deserialize your arguments, or
+    # a None in order to pass ModbusTCPMessage as argument
     0x03: (read_holding_registers, STRUCT_HH),
     0x04: (read_analog_inputs, STRUCT_HH),
     0x01: (read_coil, STRUCT_HH),
@@ -91,14 +93,10 @@ TRANSLATION_TABLE = {
 
 
 class ModbusProcessor:
-    __slots__ = ('server', 'struct_cache')
+    __slots__ = ('data_source',)
 
-    def __init__(self, server):
-        self.server = weakref.proxy(server)
-
-    @property
-    def data_source(self) -> BaseDataSource:
-        return self.server.data_source
+    def __init__(self, data_source):
+        self.data_source = data_source
 
     def process(self, msg: MODBUSTCPMessage) -> MODBUSTCPMessage:
         """
